import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;

import org.apache.poi.hssf.usermodel.HSSFSheet;			//excel-pakke: m lagre som excel 97-2003
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;



public class Problem {
	
	private ArrayList<Terminal> terminals;
	private ArrayList<Node> nodes;
	private Vessel vessel;
	private int numberOfTerminals;
	private int numberOfNodes;
	private double [][] distanceMatrix;
	private double totalDeliveryLoad; 
	private double totalPickupLoad; 
	private double totalLoad;
	private double [][] timeBetweenNodes;
	private int [][] arcs;
	private ArrayList<Node> pickupNodes;
	private ArrayList<Node> deliveryNodes; 
	private double [][] bigM;
	private int [] numberOfNodesInTerminal;
	private int [] numberOfDeliveriesInTerminal;
	private int [] numberOfPickupsInTerminal;
	
	
	
	public Problem(HSSFWorkbook workbook){				//oppretter problemet - tar ikke hensyn tanker p skipet (problem 1)
		terminals = importTerminals(workbook);
		nodes = importNodes(workbook);
		distanceMatrix = importDistances(workbook);
		vessel = setVesselInfo();
		numberOfTerminals = calcNumberOfTerminals();
		numberOfNodes = calcNumberOfNodes();
		totalDeliveryLoad = calcTotalDeliveryLoad();
		totalPickupLoad = calcTotalPickupLoad();
		totalLoad = calcTotalLoad();
		timeBetweenNodes = calcTimeBetweenNodes();
		pickupNodes = findPickupNodes();
		deliveryNodes = findDeliveryNodes(); 
		bigM = calcBigM();
		arcs = calcArcs();
		numberOfNodesInTerminal = calcNumberOfNodesInTerminal();
		numberOfDeliveriesInTerminal = calcNumberOfDeliveriesInTerminal();
		numberOfPickupsInTerminal = calcNumberOfPickupsInTerminal();
		setNeighboursInfo();
	}
	


	public ArrayList<Terminal> getTerminals() {
		return terminals;
	}

	public ArrayList<Node> getNodes() {
		return nodes;
	}

	public Vessel getVessel() {
		return vessel;
	}

	public int getNumberOfTerminals() {
		return numberOfTerminals;
	}

	public int getNumberOfNodes() {
		return numberOfNodes;
	}

	public double[][] getDistanceMatrix() {
		return distanceMatrix;
	}

	public double getTotalDeliveryLoad() {
		return totalDeliveryLoad;
	}

	public double getTotalPickupLoad() {
		return totalPickupLoad;
	}

	public double getTotalLoad() {
		return totalLoad;
	}

	public double[][] getTimeBetweenNodes() {
		return timeBetweenNodes;
	}

	public int[][] getArcs() {
		return arcs;
	}

	public ArrayList<Node> getPickupNodes() {
		return pickupNodes;
	}

	public ArrayList<Node> getDeliveryNodes() {
		return deliveryNodes;
	}

	public double[][] getBigM() {
		return bigM;
	}

	public int[] getNumberOfNodesInTerminal() {
		return numberOfNodesInTerminal;
	}
	
	public int[] getNumberOfDeliveriesInTerminal(){
		return numberOfDeliveriesInTerminal;
	}
	
	public int[] getNumberOfPickupsInTerminal(){
		return numberOfPickupsInTerminal;
	}
	

	//----------------------------Hente inn fra excel ------------------------------------


	
	public ArrayList<Terminal> importTerminals(HSSFWorkbook workbook){
		HSSFSheet terminalSheet = workbook.getSheet("TerminalInfo");
		int numberOfRows = terminalSheet.getLastRowNum(); 	// frste rad er info
		
		ArrayList<Terminal> terminals = new ArrayList<Terminal>();
		
		for (int rowNum = 1; rowNum <= numberOfRows; rowNum++){
			Row row = terminalSheet.getRow(rowNum); 					//rowNum er radnummeret, mens row er hele raden 
			
				Cell terminalName = row.getCell(0);
				terminalName.setCellType(Cell.CELL_TYPE_STRING);
				Cell terminalID = row.getCell(1);
				Cell draftLimit = row.getCell(2);
				Cell waitingTime = row.getCell(3);
				Cell enteringTime = row.getCell(4);
				
				terminals.add(new Terminal(
						(String)terminalName.getStringCellValue(),
						(int)terminalID.getNumericCellValue(),
						(double)draftLimit.getNumericCellValue(),
						(double)waitingTime.getNumericCellValue(),
						(double)enteringTime.getNumericCellValue()
						));
		}
//		System.out.println(terminals);
		return terminals;
	}

	
	public ArrayList<Node> importNodes(HSSFWorkbook workbook){
		HSSFSheet nodeSheet = workbook.getSheet("NodeInfo");
		int numberOfRows = nodeSheet.getLastRowNum(); 		

		
		ArrayList<Node> nodes = new ArrayList<Node>();
		
		for(int rowNum = 1; rowNum <= numberOfRows; rowNum++){
			Row row = nodeSheet.getRow(rowNum);	
				Cell nodeNumber = row.getCell(0);
				Cell associatedTerminalTemp = row.getCell(1);
				associatedTerminalTemp.setCellType(Cell.CELL_TYPE_STRING);
				Cell pOrD = row.getCell(2);
				Cell washingTime = row.getCell(3);
				Cell cargoLoad = row.getCell(4);
				Cell loadingTime = row.getCell(5);
				Cell lowerTimeWindow = row.getCell(6);
				Cell upperTimeWindow = row.getCell(7);
				Cell density = row.getCell(8);
				Cell cargoType = row.getCell(9);
				
				Terminal nodeTerminal = null;
				 
				for(Terminal t : terminals){
					if(t.getTerminalName().equals(associatedTerminalTemp.getStringCellValue())) {
						nodeTerminal = t;
						break;
					}
				}
				
				nodes.add(new Node(
						(int)nodeNumber.getNumericCellValue(),
						nodeTerminal,
						(int)pOrD.getNumericCellValue(),
						(double)washingTime.getNumericCellValue(),
						(double)cargoLoad.getNumericCellValue(),
						(double)loadingTime.getNumericCellValue(),
						(double)lowerTimeWindow.getNumericCellValue(),
						(double)upperTimeWindow.getNumericCellValue(),
						(double)density.getNumericCellValue(),		//trengs egentlig ikke i modell 1, men lastes inn for  slippe  ha ulike 2 node-objekter
						(int)cargoType.getNumericCellValue()));		//trengs egentlig ikke i modell 1, men lastes inn for  slippe  ha ulike 2 node-objekter
				
			}
		return nodes;
	}

	public double[][] importDistances(HSSFWorkbook workbook){
	HSSFSheet distanceSheet = workbook.getSheet("DistanceMatrix");
	int numberOfRows = distanceSheet.getLastRowNum();					//starter telling paa 0. numberOfRows = 3 hvis 4 rader


	double [ ] [ ] distances = new double [numberOfRows+1] [numberOfRows+1];
	

	for (Row row : distanceSheet) {
	      for (Cell cell : row) {
	    	  
	    	  int rowNum = row.getRowNum(); 
	    	  int colNum = cell.getColumnIndex();
	    	  
	    	  distances[rowNum][colNum] = cell.getNumericCellValue(); 	
	      }
	}
	return distances; 
}

	public static Vessel setVesselInfo(){			//Verdiene settes her, kan velge  lese de inn fra excel
		Vessel vessel = new Vessel();

		vessel.setShipSpeed(8);
		
		if(Parameters.five_or_eight_nodes){
			vessel.setShipCapacity(11000);
		}
		if(Parameters.twelve_or_fiftheen_nodes){
			vessel.setShipCapacity(19805);
		}
		if(Parameters.twenty_or_twentyfive_nodes){
			vessel.setShipCapacity(37479);
		}
		if(Parameters.thirty_nodes){
			vessel.setShipCapacity(40048);
		}
		
		return vessel;
		
	}
	
	public void setNeighboursInfo(){
		
		for(Node n : nodes){
			ArrayList<Node> nodesInSameTerm = new ArrayList<Node>();
			for(Node n2 : nodes){
				if(n.getpOrD() !=0 && n2 != n && n2.getAssociatedTerminal().getTerminalID() == n.getAssociatedTerminal().getTerminalID()){
					nodesInSameTerm.add(n2);
				}
			}
			n.setNeighbours(nodesInSameTerm);
		}		
	}
	
	
	
	public int calcNumberOfTerminals(){				//numberOfTerminals inneholder anchorage.
		int numberOfTerminals = terminals.size(); 	
		return numberOfTerminals;
	}
	
	public int calcNumberOfNodes(){					//numberOfNodes inneholder start og slutt-noden
		int numberOfNodes = nodes.size(); 	
	
		return numberOfNodes;
	}

	public double calcTotalDeliveryLoad(){  		//totalDeliveryLoad er et positivt tall
		double totalDeliveryLoad = 0;
		for(Node n : nodes) {
			if( n.getpOrD() < 0 ){
			totalDeliveryLoad = totalDeliveryLoad - n.getCargoLoad();
			}
		}
		return totalDeliveryLoad;
	}
	
	public double calcTotalPickupLoad(){
		double totalPickupLoad = 0;
		for(Node n : nodes) {
			if( n.getpOrD() > 0 ){
				totalPickupLoad = totalPickupLoad + (n.getCargoLoad());
			}
		}
		return totalPickupLoad;
	}
	
	public double calcTotalLoad(){
		double totalLoad = getTotalPickupLoad() + getTotalDeliveryLoad();
		return totalLoad;
	}
	
	public ArrayList<Node> findPickupNodes(){
		ArrayList<Node> pNodes = new ArrayList<Node>();
		
		for(Node n : nodes) {
			if(n.getpOrD()==1){
				pNodes.add(n);
			}
		}
		return pNodes;
	}
	
	public ArrayList<Node> findDeliveryNodes(){
		ArrayList<Node> dNodes = new ArrayList<Node>();
		
		for(Node n : nodes) {
			if(n.getpOrD()==-1){
				dNodes.add(n);
			}
		}
		return dNodes;
	}

	public double[][] calcTimeBetweenNodes(){
		
		double[][] sailingTime = new double[numberOfNodes][numberOfNodes]; 		//inkluderer distance/speed(i,j) + ventetid terminal tilhrende(j) + evt negativ entering time(j) dersom i og j er i samme terminal
		double[][] timeBetweenNodes = new double[numberOfNodes][numberOfNodes];	//inkluderer loadingtime(i) + washingtime(i) + sailingtime(i,j) + entering time(j)
		
		for(int i = 0; i < numberOfNodes; i++){
			for(int j = 0; j < numberOfNodes; j++){
				
				sailingTime[i][j] = ((distanceMatrix[nodes.get(i).getAssociatedTerminal().getTerminalID()][nodes.get(j).getAssociatedTerminal().getTerminalID()] / vessel.getShipSpeed() + nodes.get(j).getAssociatedTerminal().getWaitingTime()));
				
				if(nodes.get(i).getAssociatedTerminal().getTerminalID()==nodes.get(j).getAssociatedTerminal().getTerminalID()){
					sailingTime[i][j] = - (nodes.get(j).getAssociatedTerminal().getEnteringTime());
				}
				if(i==j){
					sailingTime[i][j] = 0;
				}
			}
		}
		
		
		
		for(int i = 0; i < numberOfNodes; i++){
			for(int j = 0; j < numberOfNodes; j++){
				
				if(i==j){
					timeBetweenNodes[i][j] = 0;
				}
				else{
				timeBetweenNodes[i][j] = nodes.get(i).getLoadingTime() + nodes.get(i).getWashingTime() + sailingTime[i][j] + nodes.get(j).getAssociatedTerminal().getEnteringTime();
				}
			}
		}

	return timeBetweenNodes;
	}
	

	
	
	public boolean equalOrGreaterTimeWindows(Node pickup, Node delivery){
		boolean equal = false;
		
		if(pickup.getLowerTimeWindow() >= delivery.getLowerTimeWindow() && pickup.getUpperTimeWindow() >= delivery.getUpperTimeWindow()){ 
			equal = true;
		}
		
		return equal; 
	}
	
	public boolean nodesInSameTerminal(Node n, Node n2){
		
		if(n.getAssociatedTerminal().getTerminalID()==n2.getAssociatedTerminal().getTerminalID()){
			return true;
		}
		
		return false;
	}
	
	public int[][] calcArcs(){					//ulovlige buer
		int[][] arcs = new int[numberOfNodes][numberOfNodes];
		
		for(int i = 0; i < numberOfNodes; i++){
			for(int j = 0; j < numberOfNodes; j++){
				arcs[i][j]=1;
				if(i==j){
					arcs[i][j]=0;
				}
			}
		}
		
		arcs[0][numberOfNodes-1] = 0;				//bue mellom start og sluttnode ulovlig
		arcs[numberOfNodes-1][0] = 0;
		

		for(int i = 0; i < numberOfNodes; i++){		//buer inn til startnoden ulovlig
			arcs[i][0] = 0;
		}
		
		
		for(int i = 0; i < numberOfNodes; i++){		//buer ut fra sluttnoden ulovlig
			arcs[numberOfNodes-1][i] = 0;
		}
		

		for(int i = 0; i < numberOfNodes; i++){
			for(int j = 0; j < numberOfNodes; j++){
				if(i != j){
					if((nodes.get(i).getLowerTimeWindow() + timeBetweenNodes[i][j]) > nodes.get(j).getUpperTimeWindow()){
						arcs[i][j] = 0;
					}
				}
			}
		}	
	
		for(int i = 1; i < numberOfNodes-1; i++){ 		//sjekker ikke start- og slutt-noden. 
			if(nodes.get(i).getAssociatedTerminal().getDraftLimit() < totalPickupLoad){
				arcs[i][numberOfNodes-1]=0;
			}
		}
		
		
	
		for(int j = 1; j < numberOfNodes-1; j++){ 		//sjekker ikke start- og slutt-noden. 
			if(nodes.get(j).getAssociatedTerminal().getDraftLimit() < totalDeliveryLoad){
				arcs[0][j]=0;
			}
		}

		
		for(Node n : pickupNodes){
			for (Node n2 : nodes){
				if (n2.getAssociatedTerminal().getDraftLimit() < n.getCargoLoad() && (n.getNodeNumber() != n2.getNodeNumber())){
					
					arcs[n.getNodeNumber()][n2.getNodeNumber()]=0;
				}
				if(pickupNodes.contains(n2)){
					if ((n2.getAssociatedTerminal().getDraftLimit() < (n.getCargoLoad()+n2.getCargoLoad())) && (n.getNodeNumber() != n2.getNodeNumber())){
						arcs[n.getNodeNumber()][n2.getNodeNumber()]=0;
					}
				}
			}
		}
		
		//Fjerne symmetriske lsninger: 
		
		for(Node n : nodes){
			for(Node n2 : nodes){
				if(nodesInSameTerminal(n,n2) && n.getpOrD()==1 && n2.getpOrD()==-1 && equalOrGreaterTimeWindows(n,n2) && arcs[n.getNodeNumber()][n2.getNodeNumber()]==1 && arcs[n2.getNodeNumber()][n.getNodeNumber()]==1){		//ulovlig  g fra pickup til delivery hvis i samme terminal og samme tidsvindu, og begge buene eksisterer 
					arcs[n.getNodeNumber()][n2.getNodeNumber()] = 0;
				}
			}
		}
		
		
		
		for(Terminal t : terminals){
			if(t.getTerminalID() != 0){
				ArrayList<Node> pickups = findPickups(t);
				ArrayList<Node> deliveries = findDeliveries(t);
				int numberOfNodesInT = numberOfNodesInTerminal(t);
				
				if(pickups.size() == numberOfNodesInT){
					for(Node n1 : pickups){
						for(Node n2 : pickups){
							if(arcs[n1.getNodeNumber()][n2.getNodeNumber()] == 1 && arcs[n2.getNodeNumber()][n1.getNodeNumber()] == 1){			//begge arcsene m eksistere fr vi kan utlukke en
								if(n1.getLowerTimeWindow() < n2.getLowerTimeWindow() && n1.getUpperTimeWindow() < n2.getUpperTimeWindow()){
									arcs[n2.getNodeNumber()][n1.getNodeNumber()] = 0;
								}
								else if(n1.getLowerTimeWindow() < n2.getLowerTimeWindow() && n1.getUpperTimeWindow() == n2.getUpperTimeWindow()){
									arcs[n2.getNodeNumber()][n1.getNodeNumber()] = 0;
								}
								else if(n1.getLowerTimeWindow() == n2.getLowerTimeWindow() && n1.getUpperTimeWindow() < n2.getUpperTimeWindow()){
									arcs[n2.getNodeNumber()][n1.getNodeNumber()] = 0;
								}
								else if(n1.getLowerTimeWindow() == n2.getLowerTimeWindow() && n1.getUpperTimeWindow() == n2.getUpperTimeWindow()){
									if(n1.getLoadingTime() < n2.getLoadingTime()){
										arcs[n2.getNodeNumber()][n1.getNodeNumber()] = 0;
									}
									else if(n1.getLoadingTime() == n2.getLoadingTime()){
										if(n1.getCargoLoad() < n2.getCargoLoad()){
											arcs[n2.getNodeNumber()][n1.getNodeNumber()] = 0;
										}
									}
								}
							}
						}
					}
				}
				
				else if(deliveries.size() == numberOfNodesInT){
					for(Node n1 : deliveries){
						for(Node n2 : deliveries){
							if(arcs[n1.getNodeNumber()][n2.getNodeNumber()] == 1 && arcs[n2.getNodeNumber()][n1.getNodeNumber()] == 1){			//begge arcsene m eksistere fr vi kan utlukke en
								if(n1.getLowerTimeWindow() < n2.getLowerTimeWindow() && n1.getUpperTimeWindow() < n2.getUpperTimeWindow()){
									arcs[n2.getNodeNumber()][n1.getNodeNumber()] = 0;
								}
								else if(n1.getLowerTimeWindow() < n2.getLowerTimeWindow() && n1.getUpperTimeWindow() == n2.getUpperTimeWindow()){
									arcs[n2.getNodeNumber()][n1.getNodeNumber()] = 0;
								}
								else if(n1.getLowerTimeWindow() == n2.getLowerTimeWindow() && n1.getUpperTimeWindow() < n2.getUpperTimeWindow()){
									arcs[n2.getNodeNumber()][n1.getNodeNumber()] = 0;
								}
								else if(n1.getLowerTimeWindow() == n2.getLowerTimeWindow() && n1.getUpperTimeWindow() == n2.getUpperTimeWindow()){
									if(n1.getLoadingTime() < n2.getLoadingTime()){
										arcs[n2.getNodeNumber()][n1.getNodeNumber()] = 0;
									}
									else if(n1.getLoadingTime() == n2.getLoadingTime()){
										if(n1.getCargoLoad() < n2.getCargoLoad()){
											arcs[n2.getNodeNumber()][n1.getNodeNumber()] = 0;
										}
									}
								}
							}
						}
					}
				}
				else{
					continue; 
				}
			}	
		}
		return arcs;
	}

	
	public double[][] calcBigM(){	
		double[][] bigM = new double[numberOfNodes][numberOfNodes];
		
		for(Node n : nodes){
			for (Node n2 : nodes){
				if (n.getNodeNumber()==n2.getNodeNumber()){
					bigM[n.getNodeNumber()][n2.getNodeNumber()] = 0;
				}
				else{
					double m = (n.getUpperTimeWindow() + timeBetweenNodes[n.getNodeNumber()][n2.getNodeNumber()] - n2.getLowerTimeWindow());
					
					if(m<=0){
						bigM[n.getNodeNumber()][n2.getNodeNumber()] = 0;
					}
					else{
						bigM[n.getNodeNumber()][n2.getNodeNumber()] = m;
					}
							
				}
			}
		}
		return bigM;
	}
	
	public int[] calcNumberOfNodesInTerminal(){
		
		int[] numberOfNodesInTerminal = new int[numberOfTerminals];
		
		for(Terminal t : terminals){
			for(Node n : nodes){
				if (n.getAssociatedTerminal().getTerminalID() == t.getTerminalID()){
					numberOfNodesInTerminal[t.getTerminalID()] = numberOfNodesInTerminal[t.getTerminalID()]+1;
				}
			}
		}
		
		return numberOfNodesInTerminal;
	}
	
	public int[] calcNumberOfDeliveriesInTerminal(){
		
		int[] numberOfDeliveriesInTerminal = new int[numberOfTerminals];
		
		for(Terminal t : terminals){
			for(Node n : nodes){
				if (n.getAssociatedTerminal().getTerminalID() == t.getTerminalID() && n.getpOrD() == -1){
					numberOfDeliveriesInTerminal[t.getTerminalID()] = numberOfDeliveriesInTerminal[t.getTerminalID()]+1;
				}
			}
		}
		
		return numberOfDeliveriesInTerminal;
	}
	
	public int[] calcNumberOfPickupsInTerminal(){
		
		int[] numberOfPickupsInTerminal = new int[numberOfTerminals];
		
		for(Terminal t : terminals){
			for(Node n : nodes){
				if (n.getAssociatedTerminal().getTerminalID() == t.getTerminalID() && n.getpOrD() == 1){
					numberOfPickupsInTerminal[t.getTerminalID()] = numberOfPickupsInTerminal[t.getTerminalID()]+1;
				}
			}
		}
		
		return numberOfPickupsInTerminal;
	}
	
	public int numberOfNodesInTerminal(Terminal t){
		int number = 0;
		for(Node n : nodes){
			if(n.getAssociatedTerminal()==t){
				number = number + 1;
			}
		}
		return number;
	}
	
	public ArrayList<Node> findPickups(Terminal t){
		ArrayList<Node> pickups = new ArrayList<Node>();
		
		for(Node n : nodes){
			if(n.getAssociatedTerminal()==t && n.getpOrD()==1){
				pickups.add(n);
			}
		}
		
		return pickups;
	}
	
	public ArrayList<Node> findDeliveries(Terminal t){
		ArrayList<Node> deliveries = new ArrayList<Node>();
		
		for(Node n : nodes){
			if(n.getAssociatedTerminal()==t && n.getpOrD()==-1){
				deliveries.add(n);
			}
		}
		
		return deliveries;
	}
	
	
}
	






