import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;

import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;



public class Problem2 extends Problem{	//utvider problemet - tar hensyn tanker p skipet (problem2)

	private ArrayList<Tank> tanks;
	private int [][] initialPlacements;  
	private double [][] initialVolume; 
	ArrayList<Node> deliveries;
	
	public static boolean feasibleProblem = true;			// Hvis true, s er det mulig med en feasible tank allokering fra start, og det skal skrives ut fil 
	
	
	public Problem2(HSSFWorkbook workbook, Stopwatch stopwatch){
		super(workbook);
		tanks = importTanks(workbook);
		stopwatch.stop();								//initiell allokering av lastene skal ikke pvirke kjretidene
		deliveries = getAllDeliveries();
		initialPlacements = placingCargoes();
		initialVolume = placingVolume();
		stopwatch.start();
	}

	public ArrayList<Tank> getTanks() {
		return tanks;
	}
	
	public int[][] getInitialPlacements() {
		return initialPlacements;
	}
	
	public double[][] getInitialVolume() {
		return initialVolume;
	}


		//------------------Hente inn fra excel-------------------------
		public ArrayList<Tank> importTanks(HSSFWorkbook workbook){
						
			HSSFSheet tankSheet = workbook.getSheet("VesselInfo");
			int numberOfRows = tankSheet.getLastRowNum(); 			
			
			ArrayList<Tank> tanks = new ArrayList<Tank>();
			
			for(int rowNum = 1; rowNum <= numberOfRows; rowNum++){
				Row row = tankSheet.getRow(rowNum);	
					
					Cell tankID = row.getCell(0);
					Cell minCap = row.getCell(1);
					Cell maxCap = row.getCell(2);
					Cell tankType = row.getCell(3);
					
					tanks.add(new Tank(
							(int)tankID.getNumericCellValue(),
							(double)minCap.getNumericCellValue(),
							(double)maxCap.getNumericCellValue(),
							(int)tankType.getNumericCellValue()));
					
				}
			return tanks;
		}
		

	
	public int [][] placingCargoes(){
		
		int[][] initialPlacements = new int[getNumberOfNodes()-2][tanks.size()];
		for(int i = 0; i < getNumberOfNodes()-2; i++){
			for(int j = 0; j < tanks.size(); j++){
			}
		}
		
		//deliveryloadsene skal plasseres ut p skipet
		
		
		ArrayList<Tank> tempTanks = new ArrayList<>(tanks);
		
		Collections.sort(tempTanks, new TankComparator());		//minste tank frst
		Collections.sort(deliveries, new NodeComparator());		//hyest load frst
		
		
		for(int n = 0; n < deliveries.size(); n++){
			int numberOfTanksNeededToPlaceLoad = numberOfTanksNeededToPlaceLoad(deliveries.get(n),tempTanks);
			if(numberOfTanksNeededToPlaceLoad>13){
				System.out.println("ERROR - expand code. Number of tanks a cargo needs to be distributed on: " + numberOfTanksNeededToPlaceLoad);
			}
			ArrayList<Tank> tanksToPlaceCargoOn = tanksToPlaceCargoOn(deliveries.get(n), tempTanks, numberOfTanksNeededToPlaceLoad); // Returnerer hvilke tanker som skal brukes
			if(tanksToPlaceCargoOn == null ) {
				System.out.println("ERROR");
			}
			if(numberOfTanksNeededToPlaceLoad==0){
				//System.out.println("tanksToPlaceCargoOn = NULL for cargo:" + deliveries.get(n).getNodeID());
				System.out.println("ERROR: not feasible initialization");
				return null;
			}
			
			for(int i = 0; i < numberOfTanksNeededToPlaceLoad; i++){
				initialPlacements[deliveries.get(n).getNodeNumber()-1][tanksToPlaceCargoOn.get(i).getTankID()-1] = 1;  //m trekke fra 1 for at matrisen skal vre 1-indeksert. get0: kun ett element dersom man bare trenger en tank
				tempTanks.remove(tanksToPlaceCargoOn.get(i)); // M fjerne den tanken som er blitt brukt 
			}
		}

		return initialPlacements;
	}
	
	public double [][] placingVolume(){
		
		double[][] initialVolume = new double[getNumberOfNodes()-2][tanks.size()];
		for(int i = 0; i < getNumberOfNodes()-2; i++){
			for(int j = 0; j < tanks.size(); j++){
				initialVolume[i][j] = 0; 
			}
		}
		
			
		if(initialPlacements != null){
			
			//plasserer cargoene jevnt dersom de er plassert p flere tanker
			
			for(Node n : deliveries){
				
				int noOfTanksNeeded = numberOfTanksCargoIsPlacedOn(n, initialPlacements);
				
				double totalLoadToPlaceOut = -n.getCargoLoadVolume(); 
				
				if(Parameters.nothingIncompatible){
					
					ArrayList<Tank> tanksNeeded = new ArrayList<>(tanks);
					Collections.sort(tanksNeeded, new TankComparator());
					
					double loadToPlaceOut = totalLoadToPlaceOut/noOfTanksNeeded;
					
					for(int t = 0; t < tanksNeeded.size(); t++){
						
						for(int i = 0; i < tanks.size(); i++){
							
							if(initialPlacements[n.getNodeNumber()-1][i] == 1){
								
								if(tanksNeeded.get(t).getTankID()==tanks.get(i).getTankID()){
								
									initialVolume[n.getNodeNumber()-1][i] = Math.min(loadToPlaceOut, tanksNeeded.get(t).getMaxCap());
									totalLoadToPlaceOut = totalLoadToPlaceOut - Math.min(loadToPlaceOut, tanksNeeded.get(t).getMaxCap());
									noOfTanksNeeded = noOfTanksNeeded -1;
									loadToPlaceOut = totalLoadToPlaceOut/noOfTanksNeeded;
									
								}
							}
						}
					}
					
				}
				
				else{			// Vi har inkompatibilitet
					
					ArrayList<Tank> tanksNeeded = new ArrayList<>();
					
					for(int t = 0; t < tanks.size();t++){
						if(n.getCargoType()==tanks.get(t).getTankType() || n.getCargoType() == 3){
							tanksNeeded.add(tanks.get(t));
						}
					}
					
					Collections.sort(tanksNeeded, new TankComparator()); // Minste tank frst
					
					double loadToPlaceOut = totalLoadToPlaceOut/noOfTanksNeeded;  //fordeler de jevnt
					
					
					for(int t = 0; t < tanksNeeded.size(); t++){
						
							for(int i = 0; i < tanks.size(); i++){
								
								if(initialPlacements[n.getNodeNumber()-1][i] == 1){
									
									if(tanksNeeded.get(t).getTankID()==tanks.get(i).getTankID()){
									
										initialVolume[n.getNodeNumber()-1][i] = Math.min(loadToPlaceOut, tanksNeeded.get(t).getMaxCap());
										totalLoadToPlaceOut = totalLoadToPlaceOut - Math.min(loadToPlaceOut, tanksNeeded.get(t).getMaxCap());
										noOfTanksNeeded = noOfTanksNeeded -1;
										loadToPlaceOut = totalLoadToPlaceOut/noOfTanksNeeded;
									}
								
								}
							
							}
					}
				}
				
					
				
			}
		}

			
		return initialVolume;
	}
	
	public int numberOfTanksCargoIsPlacedOn(Node cargo, int[][] placements){
		int number = 0;
		for(int i = 0; i < tanks.size(); i++){
			if(placements[cargo.getNodeNumber()-1][i] == 1){
				number = number+1;
			}
		}
		return number;
	}
	
	public ArrayList<Tank> tanksToPlaceCargoOn(Node cargo, ArrayList<Tank> tanks, int numberOfTanksNeeded){ //finner den minste tanken som rommer hele cargoen - returnerer NULL dersom ingen tanker kan ta den
		ArrayList<Tank> tanksToUse = new ArrayList<Tank>();
		
		if(Parameters.nothingIncompatible){
			
			if(numberOfTanksNeeded == 1){
				for(Tank t : tanks){
					if((-cargo.getCargoLoadVolume()) <= t.getMaxCap()){  
						tanksToUse.add(t);
						return tanksToUse;
					}
				}
			}
			else if(numberOfTanksNeeded == 2){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					for(Tank t2 : tanks){
						double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap();
						if(t1 != t2 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
							bestTanks.add(t1);
							bestTanks.add(t2);
							minCapacityOfTanks = capacityOfTheseTanks;
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				return tanksToUse;
			}
			else if(numberOfTanksNeeded == 3){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					for(Tank t2 : tanks){
						for(Tank t3 : tanks){
							double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap();
							if(t1 != t2 && t2 != t3 && t1!= t3 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
								bestTanks.add(t1);
								bestTanks.add(t2);
								bestTanks.add(t3);
								minCapacityOfTanks = capacityOfTheseTanks;
							}
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				
				
				return tanksToUse;
			}
			else if(numberOfTanksNeeded == 4){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					for(Tank t2 : tanks){
						for(Tank t3 : tanks){
							for(Tank t4 : tanks){
								double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap();
								if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
									bestTanks.add(t1);
									bestTanks.add(t2);
									bestTanks.add(t3);
									bestTanks.add(t4);
									minCapacityOfTanks = capacityOfTheseTanks;
								}
							}
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				
				
				return tanksToUse;
			}
			else if(numberOfTanksNeeded == 5){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					for(Tank t2 : tanks){
						for(Tank t3 : tanks){
							for(Tank t4 : tanks){
								for(Tank t5 : tanks){
									double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap()+t5.getMaxCap();
									if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && t1 != t5 && t2 != t5 && t3!= t5 && t4 != t5 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
										bestTanks.add(t1);
										bestTanks.add(t2);
										bestTanks.add(t3);
										bestTanks.add(t4);
										bestTanks.add(t5);
										minCapacityOfTanks = capacityOfTheseTanks;
									}
								}
							}
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				tanksToUse.add(bestTanks.get(bestTanks.size()-5));
				
				return tanksToUse;
			}
			else if(numberOfTanksNeeded == 6){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					for(Tank t2 : tanks){
						for(Tank t3 : tanks){
							for(Tank t4 : tanks){
								for(Tank t5 : tanks){
									for(Tank t6 : tanks){
										double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap()+t5.getMaxCap()+t6.getMaxCap();
										if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && t1 != t5 && t2 != t5 && t3!= t5 && t4 != t5 && t1 != t6 && t2 != t6 && t3!= t6 && t4 != t6 && t5 != t6 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
											bestTanks.add(t1);
											bestTanks.add(t2);
											bestTanks.add(t3);
											bestTanks.add(t4);
											bestTanks.add(t5);
											bestTanks.add(t6);
											minCapacityOfTanks = capacityOfTheseTanks;
										}
									}
								}
							}
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				tanksToUse.add(bestTanks.get(bestTanks.size()-5));
				tanksToUse.add(bestTanks.get(bestTanks.size()-6));
				
				return tanksToUse;
			}
			else if(numberOfTanksNeeded == 7){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					for(Tank t2 : tanks){
						for(Tank t3 : tanks){
							for(Tank t4 : tanks){
								for(Tank t5 : tanks){
									for(Tank t6 : tanks){
										for(Tank t7 : tanks){
											double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap()+t5.getMaxCap() + t6.getMaxCap() + t7.getMaxCap();
											if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && t1 != t5 && t2 != t5 && t3!= t5 && t4 != t5 && t1 != t6 && t2 != t6 && t3!= t6 && t4 != t6 && t5 != t6 && t1 != t7 && t2 != t7 && t3!= t7 && t4 != t7 && t5 != t7 && t6 != t7 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
												bestTanks.add(t1);
												bestTanks.add(t2);
												bestTanks.add(t3);
												bestTanks.add(t4);
												bestTanks.add(t5);
												bestTanks.add(t6);
												bestTanks.add(t7);
												minCapacityOfTanks = capacityOfTheseTanks;
											}
										}
									}
								}
							}
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				tanksToUse.add(bestTanks.get(bestTanks.size()-5));
				tanksToUse.add(bestTanks.get(bestTanks.size()-6));
				tanksToUse.add(bestTanks.get(bestTanks.size()-7));
				
				return tanksToUse;
			}
			else if(numberOfTanksNeeded == 8){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					for(Tank t2 : tanks){
						for(Tank t3 : tanks){
							for(Tank t4 : tanks){
								for(Tank t5 : tanks){
									for(Tank t6 : tanks){
										for(Tank t7 : tanks){
											for(Tank t8 : tanks){
												double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap()+t5.getMaxCap() + t6.getMaxCap() + t7.getMaxCap()+ t8.getMaxCap();
												if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && t1 != t5 && t2 != t5 && t3!= t5 && t4 != t5 && t1 != t6 && t2 != t6 && t3!= t6 && t4 != t6 && t5 != t6 && t1 != t7 && t2 != t7 && t3!= t7 && t4 != t7 && t5 != t7 && t6 != t7 && t1 != t8 && t2 != t8 && t3!= t8 && t4 != t8 && t5 != t8 && t6 != t8 && t7 != t8 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
													bestTanks.add(t1);
													bestTanks.add(t2);
													bestTanks.add(t3);
													bestTanks.add(t4);
													bestTanks.add(t5);
													bestTanks.add(t6);
													bestTanks.add(t7);
													bestTanks.add(t8);
													minCapacityOfTanks = capacityOfTheseTanks;
												}
											}
										}
									}
								}
							}
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				tanksToUse.add(bestTanks.get(bestTanks.size()-5));
				tanksToUse.add(bestTanks.get(bestTanks.size()-6));
				tanksToUse.add(bestTanks.get(bestTanks.size()-7));
				tanksToUse.add(bestTanks.get(bestTanks.size()-8));
				
				return tanksToUse;
			}
			else if(numberOfTanksNeeded == 9){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					for(Tank t2 : tanks){
						for(Tank t3 : tanks){
							for(Tank t4 : tanks){
								for(Tank t5 : tanks){
									for(Tank t6 : tanks){
										for(Tank t7 : tanks){
											for(Tank t8 : tanks){
												for(Tank t9 : tanks){
													double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap()+t5.getMaxCap() + t6.getMaxCap() + t7.getMaxCap()+ t8.getMaxCap() + t9.getMaxCap();
													if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && t1 != t5 && t2 != t5 && t3!= t5 && t4 != t5 && t1 != t6 && t2 != t6 && t3!= t6 && t4 != t6 && t5 != t6 && t1 != t7 && t2 != t7 && t3!= t7 && t4 != t7 && t5 != t7 && t6 != t7 && t1 != t8 && t2 != t8 && t3!= t8 && t4 != t8 && t5 != t8 && t6 != t8 && t7 != t8 && t1 != t9 && t2 != t9 && t3!= t9 && t4 != t9 && t5 != t9 && t6 != t9 && t7 != t9 && t8 != t9 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
														bestTanks.add(t1);
														bestTanks.add(t2);
														bestTanks.add(t3);
														bestTanks.add(t4);
														bestTanks.add(t5);
														bestTanks.add(t6);
														bestTanks.add(t7);
														bestTanks.add(t8);
														bestTanks.add(t9);
														minCapacityOfTanks = capacityOfTheseTanks;
													}
												}
												
											}
										}
									}
								}
							}
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				tanksToUse.add(bestTanks.get(bestTanks.size()-5));
				tanksToUse.add(bestTanks.get(bestTanks.size()-6));
				tanksToUse.add(bestTanks.get(bestTanks.size()-7));
				tanksToUse.add(bestTanks.get(bestTanks.size()-8));
				tanksToUse.add(bestTanks.get(bestTanks.size()-9));
				
				return tanksToUse;
			}
			else if(numberOfTanksNeeded == 10){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					for(Tank t2 : tanks){
						for(Tank t3 : tanks){
							for(Tank t4 : tanks){
								for(Tank t5 : tanks){
									for(Tank t6 : tanks){
										for(Tank t7 : tanks){
											for(Tank t8 : tanks){
												for(Tank t9 : tanks){
													for(Tank t10 : tanks){
														double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap()+t5.getMaxCap() + t6.getMaxCap() + t7.getMaxCap()+ t8.getMaxCap() + t9.getMaxCap() + t10.getMaxCap();
														if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && t1 != t5 && t2 != t5 && t3!= t5 && t4 != t5 && t1 != t6 && t2 != t6 && t3!= t6 && t4 != t6 && t5 != t6 && t1 != t7 && t2 != t7 && t3!= t7 && t4 != t7 && t5 != t7 && t6 != t7 && t1 != t8 && t2 != t8 && t3!= t8 && t4 != t8 && t5 != t8 && t6 != t8 && t7 != t8 && t1 != t9 && t2 != t9 && t3!= t9 && t4 != t9 && t5 != t9 && t6 != t9 && t7 != t9 && t8 != t9 && t1 != t10 && t2 != t10 && t3!= t10 && t4 != t10 && t5 != t10 && t6 != t10 && t7 != t10 && t8 != t10 && t9 != t10 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
															bestTanks.add(t1);
															bestTanks.add(t2);
															bestTanks.add(t3);
															bestTanks.add(t4);
															bestTanks.add(t5);
															bestTanks.add(t6);
															bestTanks.add(t7);
															bestTanks.add(t8);
															bestTanks.add(t9);
															bestTanks.add(t10);
															minCapacityOfTanks = capacityOfTheseTanks;
														}
													}
													
												}
												
											}
										}
									}
								}
							}
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				tanksToUse.add(bestTanks.get(bestTanks.size()-5));
				tanksToUse.add(bestTanks.get(bestTanks.size()-6));
				tanksToUse.add(bestTanks.get(bestTanks.size()-7));
				tanksToUse.add(bestTanks.get(bestTanks.size()-8));
				tanksToUse.add(bestTanks.get(bestTanks.size()-9));
				tanksToUse.add(bestTanks.get(bestTanks.size()-10));
				
				return tanksToUse;
			}
			else if(numberOfTanksNeeded == 11){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					for(Tank t2 : tanks){
						for(Tank t3 : tanks){
							for(Tank t4 : tanks){
								for(Tank t5 : tanks){
									for(Tank t6 : tanks){
										for(Tank t7 : tanks){
											for(Tank t8 : tanks){
												for(Tank t9 : tanks){
													for(Tank t10 : tanks){
														for(Tank t11 : tanks){
															double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap()+t5.getMaxCap() + t6.getMaxCap() + t7.getMaxCap()+ t8.getMaxCap() + t9.getMaxCap() + t10.getMaxCap() + t11.getMaxCap();
															if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && t1 != t5 && t2 != t5 && t3!= t5 && t4 != t5 && t1 != t6 && t2 != t6 && t3!= t6 && t4 != t6 && t5 != t6 && t1 != t7 && t2 != t7 && t3!= t7 && t4 != t7 && t5 != t7 && t6 != t7 && t1 != t8 && t2 != t8 && t3!= t8 && t4 != t8 && t5 != t8 && t6 != t8 && t7 != t8 && t1 != t9 && t2 != t9 && t3!= t9 && t4 != t9 && t5 != t9 && t6 != t9 && t7 != t9 && t8 != t9 && t1 != t10 && t2 != t10 && t3!= t10 && t4 != t10 && t5 != t10 && t6 != t10 && t7 != t10 && t8 != t10 && t9 != t10 && t1 != t11 && t2 != t11 && t3!= t11 && t4 != t11 && t5 != t11 && t6 != t11 && t7 != t11 && t8 != t11 && t9 != t11 && t10 != t11 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
																bestTanks.add(t1);
																bestTanks.add(t2);
																bestTanks.add(t3);
																bestTanks.add(t4);
																bestTanks.add(t5);
																bestTanks.add(t6);
																bestTanks.add(t7);
																bestTanks.add(t8);
																bestTanks.add(t9);
																bestTanks.add(t10);
																bestTanks.add(t11);
																minCapacityOfTanks = capacityOfTheseTanks;
															}
														}
														
													}
													
												}
												
											}
										}
									}
								}
							}
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				tanksToUse.add(bestTanks.get(bestTanks.size()-5));
				tanksToUse.add(bestTanks.get(bestTanks.size()-6));
				tanksToUse.add(bestTanks.get(bestTanks.size()-7));
				tanksToUse.add(bestTanks.get(bestTanks.size()-8));
				tanksToUse.add(bestTanks.get(bestTanks.size()-9));
				tanksToUse.add(bestTanks.get(bestTanks.size()-10));
				tanksToUse.add(bestTanks.get(bestTanks.size()-11));
				
				return tanksToUse;
			}
			
			else if(numberOfTanksNeeded == 12){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					for(Tank t2 : tanks){
						for(Tank t3 : tanks){
							for(Tank t4 : tanks){
								for(Tank t5 : tanks){
									for(Tank t6 : tanks){
										for(Tank t7 : tanks){
											for(Tank t8 : tanks){
												for(Tank t9 : tanks){
													for(Tank t10 : tanks){
														for(Tank t11 : tanks){
															for(Tank t12 : tanks){
																double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap()+t5.getMaxCap() + t6.getMaxCap() + t7.getMaxCap()+ t8.getMaxCap() + t9.getMaxCap() + t10.getMaxCap() + t11.getMaxCap() + t12.getMaxCap();
																if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && t1 != t5 && t2 != t5 && t3!= t5 && t4 != t5 && t1 != t6 && t2 != t6 && t3!= t6 && t4 != t6 && t5 != t6 && t1 != t7 && t2 != t7 && t3!= t7 && t4 != t7 && t5 != t7 && t6 != t7 && t1 != t8 && t2 != t8 && t3!= t8 && t4 != t8 && t5 != t8 && t6 != t8 && t7 != t8 && t1 != t9 && t2 != t9 && t3!= t9 && t4 != t9 && t5 != t9 && t6 != t9 && t7 != t9 && t8 != t9 && t1 != t10 && t2 != t10 && t3!= t10 && t4 != t10 && t5 != t10 && t6 != t10 && t7 != t10 && t8 != t10 && t9 != t10 && t1 != t11 && t2 != t11 && t3!= t11 && t4 != t11 && t5 != t11 && t6 != t11 && t7 != t11 && t8 != t11 && t9 != t11 && t10 != t11 && t1 != t12 && t2 != t12 && t3!= t12 && t4 != t12 && t5 != t12 && t6 != t12 && t7 != t12 && t8 != t12 && t9 != t12 && t10 != t12 && t11 != t12 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
																	bestTanks.add(t1);
																	bestTanks.add(t2);
																	bestTanks.add(t3);
																	bestTanks.add(t4);
																	bestTanks.add(t5);
																	bestTanks.add(t6);
																	bestTanks.add(t7);
																	bestTanks.add(t8);
																	bestTanks.add(t9);
																	bestTanks.add(t10);
																	bestTanks.add(t11);
																	bestTanks.add(t12);
																	minCapacityOfTanks = capacityOfTheseTanks;
																}
															}
															
														}
														
													}
													
												}
												
											}
										}
									}
								}
							}
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				tanksToUse.add(bestTanks.get(bestTanks.size()-5));
				tanksToUse.add(bestTanks.get(bestTanks.size()-6));
				tanksToUse.add(bestTanks.get(bestTanks.size()-7));
				tanksToUse.add(bestTanks.get(bestTanks.size()-8));
				tanksToUse.add(bestTanks.get(bestTanks.size()-9));
				tanksToUse.add(bestTanks.get(bestTanks.size()-10));
				tanksToUse.add(bestTanks.get(bestTanks.size()-11));
				tanksToUse.add(bestTanks.get(bestTanks.size()-12));
				
				return tanksToUse;
			}
			
			else if(numberOfTanksNeeded == 13){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					for(Tank t2 : tanks){
						for(Tank t3 : tanks){
							for(Tank t4 : tanks){
								for(Tank t5 : tanks){
									for(Tank t6 : tanks){
										for(Tank t7 : tanks){
											for(Tank t8 : tanks){
												for(Tank t9 : tanks){
													for(Tank t10 : tanks){
														for(Tank t11 : tanks){
															for(Tank t12 : tanks){
																for(Tank t13 : tanks){
																	double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap()+t5.getMaxCap() + t6.getMaxCap() + t7.getMaxCap()+ t8.getMaxCap() + t9.getMaxCap() + t10.getMaxCap() + t11.getMaxCap() + t12.getMaxCap() + t13.getMaxCap();
																	if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && t1 != t5 && t2 != t5 && t3!= t5 && t4 != t5 && t1 != t6 && t2 != t6 && t3!= t6 && t4 != t6 && t5 != t6 && t1 != t7 && t2 != t7 && t3!= t7 && t4 != t7 && t5 != t7 && t6 != t7 && t1 != t8 && t2 != t8 && t3!= t8 && t4 != t8 && t5 != t8 && t6 != t8 && t7 != t8 && t1 != t9 && t2 != t9 && t3!= t9 && t4 != t9 && t5 != t9 && t6 != t9 && t7 != t9 && t8 != t9 && t1 != t10 && t2 != t10 && t3!= t10 && t4 != t10 && t5 != t10 && t6 != t10 && t7 != t10 && t8 != t10 && t9 != t10 && t1 != t11 && t2 != t11 && t3!= t11 && t4 != t11 && t5 != t11 && t6 != t11 && t7 != t11 && t8 != t11 && t9 != t11 && t10 != t11 && t1 != t12 && t2 != t12 && t3!= t12 && t4 != t12 && t5 != t12 && t6 != t12 && t7 != t12 && t8 != t12 && t9 != t12 && t10 != t12 && t11 != t12 && t1 != t13 && t2 != t13 && t3!= t13 && t4 != t13 && t5 != t13 && t6 != t13 && t7 != t13 && t8 != t13 && t9 != t13 && t10 != t13 && t11 != t13 && t12 != t13 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
																		bestTanks.add(t1);
																		bestTanks.add(t2);
																		bestTanks.add(t3);
																		bestTanks.add(t4);
																		bestTanks.add(t5);
																		bestTanks.add(t6);
																		bestTanks.add(t7);
																		bestTanks.add(t8);
																		bestTanks.add(t9);
																		bestTanks.add(t10);
																		bestTanks.add(t11);
																		bestTanks.add(t12);
																		bestTanks.add(t13);
																		minCapacityOfTanks = capacityOfTheseTanks;
																	}
																}
																
															}
															
														}
														
													}
													
												}
												
											}
										}
									}
								}
							}
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				tanksToUse.add(bestTanks.get(bestTanks.size()-5));
				tanksToUse.add(bestTanks.get(bestTanks.size()-6));
				tanksToUse.add(bestTanks.get(bestTanks.size()-7));
				tanksToUse.add(bestTanks.get(bestTanks.size()-8));
				tanksToUse.add(bestTanks.get(bestTanks.size()-9));
				tanksToUse.add(bestTanks.get(bestTanks.size()-10));
				tanksToUse.add(bestTanks.get(bestTanks.size()-11));
				tanksToUse.add(bestTanks.get(bestTanks.size()-12));
				tanksToUse.add(bestTanks.get(bestTanks.size()-13));
				
				return tanksToUse;
			}
			
			
		}
		
		else{  // Hvis inkompatibilitet 
			
			if(numberOfTanksNeeded == 1){
				for(Tank t : tanks){
					if(t.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
						if((-cargo.getCargoLoadVolume()) <= t.getMaxCap()){  
							tanksToUse.add(t);
							return tanksToUse;
						}
					}
				}
			}
			else if(numberOfTanksNeeded == 2){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					if(t1.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
						for(Tank t2 : tanks){
							if(t2.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
								double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap();
								if(t1 != t2 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
									bestTanks.add(t1);
									bestTanks.add(t2);
									minCapacityOfTanks = capacityOfTheseTanks;
								}
							}
						}
					}
					
				}
				
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				return tanksToUse;
				
			}
			else if(numberOfTanksNeeded == 3){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					if(t1.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
						for(Tank t2 : tanks){
							if(t2.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
								for(Tank t3 : tanks){
									if(t3.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
										double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap();
										if(t1 != t2 && t2 != t3 && t1!= t3 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
											bestTanks.add(t1);
											bestTanks.add(t2);
											bestTanks.add(t3);
											minCapacityOfTanks = capacityOfTheseTanks;
										}
									}
								}
							}
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				
				return tanksToUse;
			}
			else if(numberOfTanksNeeded == 4){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					if(t1.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
						for(Tank t2 : tanks){
							if(t2.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
								for(Tank t3 : tanks){
									if(t3.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
										for(Tank t4 : tanks){
											if(t4.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
												double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap();
												if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
													bestTanks.add(t1);
													bestTanks.add(t2);
													bestTanks.add(t3);
													bestTanks.add(t4);
													minCapacityOfTanks = capacityOfTheseTanks;
												}
												
											}
										}
											
								
									}
								}
							}
						}
					}
				}
		
	

				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				
				
				return tanksToUse;
			}
			
			else if(numberOfTanksNeeded == 5){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					if(t1.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
						for(Tank t2 : tanks){
							if(t2.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
								for(Tank t3 : tanks){
									if(t3.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
										for(Tank t4 : tanks){
											if(t4.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
												for(Tank t5 : tanks){
													if(t5.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
														double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap()+t5.getMaxCap();
														if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && t1 != t5 && t2 != t5 && t3!= t5 && t4 != t5 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
															bestTanks.add(t1);
															bestTanks.add(t2);
															bestTanks.add(t3);
															bestTanks.add(t4);
															bestTanks.add(t5);
															minCapacityOfTanks = capacityOfTheseTanks;
														}
													}
												}
													
											}
								
										}
									}
								}
							}
						}
					}
				}
				
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				tanksToUse.add(bestTanks.get(bestTanks.size()-5));
				
				return tanksToUse;
			}
				
			else if(numberOfTanksNeeded == 6){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					if(t1.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
						for(Tank t2 : tanks){
							if(t2.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
								for(Tank t3 : tanks){
									if(t3.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
										for(Tank t4 : tanks){
											if(t4.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
												for(Tank t5 : tanks){
													if(t5.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
														for(Tank t6 : tanks){
															if(t6.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
																double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap()+t5.getMaxCap()+t6.getMaxCap();
																if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && t1 != t5 && t2 != t5 && t3!= t5 && t4 != t5 && t1 != t6 && t2 != t6 && t3!= t6 && t4 != t6 && t5 != t6 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
																	bestTanks.add(t1);
																	bestTanks.add(t2);
																	bestTanks.add(t3);
																	bestTanks.add(t4);
																	bestTanks.add(t5);
																	bestTanks.add(t6);
																	minCapacityOfTanks = capacityOfTheseTanks;
																}
															}
														}
															
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
				
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				tanksToUse.add(bestTanks.get(bestTanks.size()-5));
				tanksToUse.add(bestTanks.get(bestTanks.size()-6));
				
				return tanksToUse;
			}
			
			else if(numberOfTanksNeeded == 7){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					if(t1.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
						for(Tank t2 : tanks){
							if(t2.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
								for(Tank t3 : tanks){
									if(t3.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
										for(Tank t4 : tanks){
											if(t4.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
												for(Tank t5 : tanks){
													if(t5.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
														for(Tank t6 : tanks){
															if(t6.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
																for(Tank t7 : tanks){
																	if(t7.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
																		double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap()+t5.getMaxCap() + t6.getMaxCap() + t7.getMaxCap();
																		if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && t1 != t5 && t2 != t5 && t3!= t5 && t4 != t5 && t1 != t6 && t2 != t6 && t3!= t6 && t4 != t6 && t5 != t6 && t1 != t7 && t2 != t7 && t3!= t7 && t4 != t7 && t5 != t7 && t6 != t7 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
																			bestTanks.add(t1);
																			bestTanks.add(t2);
																			bestTanks.add(t3);
																			bestTanks.add(t4);
																			bestTanks.add(t5);
																			bestTanks.add(t6);
																			bestTanks.add(t7);
																			minCapacityOfTanks = capacityOfTheseTanks;
																		}
																	}
																}
																	
															}
														}
													}
												}
											}
										}
									}
								}
							}
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				tanksToUse.add(bestTanks.get(bestTanks.size()-5));
				tanksToUse.add(bestTanks.get(bestTanks.size()-6));
				tanksToUse.add(bestTanks.get(bestTanks.size()-7));
				
				return tanksToUse;
			}
			
			else if(numberOfTanksNeeded == 8){
				double minCapacityOfTanks = getVessel().getShipCapacity(); //et hyt tall
				ArrayList<Tank> bestTanks = new ArrayList<Tank>();
				
				for(Tank t1 : tanks){
					if(t1.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
						for(Tank t2 : tanks){
							if(t2.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
								for(Tank t3 : tanks){
									if(t3.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
										for(Tank t4 : tanks){
											if(t4.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
												for(Tank t5 : tanks){
													if(t5.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
														for(Tank t6 : tanks){
															if(t6.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
																for(Tank t7 : tanks){
																	if(t7.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
																		for(Tank t8 : tanks){
																			if(t8.getTankType()==cargo.getCargoType() || cargo.getCargoType() == 3){
																				double capacityOfTheseTanks = t1.getMaxCap()+ t2.getMaxCap()+t3.getMaxCap()+t4.getMaxCap()+t5.getMaxCap() + t6.getMaxCap() + t7.getMaxCap()+ t8.getMaxCap();
																				if(t1 != t2 && t2 != t3 && t1!= t3 && t1 != t4 && t2 != t4 && t3!= t4 && t1 != t5 && t2 != t5 && t3!= t5 && t4 != t5 && t1 != t6 && t2 != t6 && t3!= t6 && t4 != t6 && t5 != t6 && t1 != t7 && t2 != t7 && t3!= t7 && t4 != t7 && t5 != t7 && t6 != t7 && t1 != t8 && t2 != t8 && t3!= t8 && t4 != t8 && t5 != t8 && t6 != t8 && t7 != t8 && capacityOfTheseTanks > (-cargo.getCargoLoadVolume()) && capacityOfTheseTanks < minCapacityOfTanks){ //hvis det gr  plassere de her og det er ingen andre kombinasjoner som bruker mindre kapasitet.. 
																					bestTanks.add(t1);
																					bestTanks.add(t2);
																					bestTanks.add(t3);
																					bestTanks.add(t4);
																					bestTanks.add(t5);
																					bestTanks.add(t6);
																					bestTanks.add(t7);
																					bestTanks.add(t8);
																					minCapacityOfTanks = capacityOfTheseTanks;
																				}
																			}
																		}
																			
																	}
																}
															}
														}
													}
												}
											}
											
										}
									}
								}
							}
						}
					}
				}
				tanksToUse.add(bestTanks.get(bestTanks.size()-1));
				tanksToUse.add(bestTanks.get(bestTanks.size()-2));
				tanksToUse.add(bestTanks.get(bestTanks.size()-3));
				tanksToUse.add(bestTanks.get(bestTanks.size()-4));
				tanksToUse.add(bestTanks.get(bestTanks.size()-5));
				tanksToUse.add(bestTanks.get(bestTanks.size()-6));
				tanksToUse.add(bestTanks.get(bestTanks.size()-7));
				tanksToUse.add(bestTanks.get(bestTanks.size()-8));
				
				return tanksToUse;
			}
			
		}
		

		return null;  //Dette vil vel aldri skje pga funksjonen som kjres fr denne?
	}
	
	public ArrayList<Node> getAllDeliveries(){
		ArrayList<Node> deliveries = new ArrayList<Node>();
		for(Node n: getNodes()){
			if(n.getpOrD()==-1){
				deliveries.add(n);
			}
		}
		return deliveries;
	}

	public int numberOfTanksNeededToPlaceLoad(Node cargo, ArrayList<Tank> tanks){
		
		int numberOfTanks = 0;
		double loadPlaced = 0;
		double loadToPlace = -cargo.getCargoLoadVolume();
		
		if(Parameters.nothingIncompatible){  // alt er lov 
			
			for(int t = (tanks.size()-1); t >=0; t--){  //sjekker de strste tankene frst
				if(loadToPlace < tanks.get(t).getMaxCap()){
					numberOfTanks = numberOfTanks +1;
					loadToPlace = 0;
					
					break;
				}
				else{
					loadPlaced = loadPlaced + tanks.get(t).getMaxCap();
					loadToPlace = loadToPlace - tanks.get(t).getMaxCap();
					numberOfTanks = numberOfTanks +1;
				}
			
			}	
			
			
			if(loadToPlace != 0){		//har ikke nok tanker til  plassere cargo. Problemet er infeasible. Returner 0. 
				System.out.println("LoadPlaced: " + loadPlaced + " LoadToPlace: " + loadToPlace + " CargoNR: " + cargo.getNodeNumber());
				System.out.println("Not enough tanks to initally allocate cargoes. The problem is infeasible.");
				feasibleProblem=false;
				return 0; 
			}
			
		}
		
		else{     // Vi har inkompatibilitet 
			
			for(int t = (tanks.size()-1); t >=0; t--){  //sjekker de strste tankene frst
				if(cargo.getCargoType()==tanks.get(t).getTankType() || cargo.getCargoType() == 3){   // kan kun sjekke de tankene som er kompatible med cargoen 
					
					if(loadToPlace < tanks.get(t).getMaxCap()){
						numberOfTanks = numberOfTanks +1;
						loadToPlace = 0;
						
						break;
					}
					else{
						loadPlaced = loadPlaced + tanks.get(t).getMaxCap(); // Trenger vi denne? 
						loadToPlace = loadToPlace - tanks.get(t).getMaxCap();
						numberOfTanks = numberOfTanks +1;
					}
				}
			
			}	
			
			if(loadToPlace != 0){		//har ikke nok tanker til  plassere cargo. Problemet er infeasible. Returner 0. 
				System.out.println("LoadPlaced: " + loadPlaced + " LoadToPlace: " + loadToPlace + " CargoNR: " + cargo.getNodeNumber());
				System.out.println("Not enough tanks to initally allocate cargoes. The problem is infeasible.");
				feasibleProblem=false;
				return 0; 
			}
			
		}
	
		
		
		
		return numberOfTanks;
	}
}
